# Load package for parallel computing
require('multicore')

## FUNCTIONS 

# R(x)
resources <- function(x, prms){
	Rx <- (prms$q*exp(-(x-prms$deltatheta/2)^2/(2*prms$sigmaR^2)) + (1-prms$q)*exp(-(x+prms$deltatheta/2)^2/(2*prms$sigmaR^2)))
	return(Rx/sum(Rx))
}

# Predicted distribution at equilibrium
distrib <- function(x, prms){
	Dx <- (prms$q*exp(-(x-prms$deltatheta/2)^2/(2*(prms$sigmaR^2- prms$sigmaS^2))) + (1-prms$q)*exp(-(x+prms$deltatheta/2)^2/(2*(prms$sigmaR^2- prms$sigmaS^2))))
	return(Dx/sum(Dx))
}

# Whether the predicted distribution of resources is bimodal
#   Evaluate this by looking at how many times the sign of the slope changes
is.bimod <- function(prms, thr=0, rnd=3){
	x <- seq(-prms$deltatheta/2, prms$deltatheta/2, length.out=prms$npts);
	dist <- distrib(x, prms)
	if( sum(diff(diff(dist)<0)!=0)==1 && dist[2]-dist[1]>0){out <- 0}else{out <- 1};
	return(out)
}

## GLOBAL PARAMETERS 
cores <- 2;	# Number of cores that will be used

# Whether the genetic architecture is constrained
theconstrain <- FALSE;

# Arrays of parameters
# -> There will be ndelta*nsigmaS simulations

# \Delta \theta
ndelta <- 250;	# Number of possible values
dmax <- 0.4;
dmin <- 0.001;
deltalist <- seq(dmin, dmax, length.out=ndelta);

# \sigma_S
sigmaSlist <- c(0.1);

# \sigma_R
nsigmaR <- 250;	# Number of possible values
sRmax <- 0.25;
sRmin <- 0.1+0.00001;
invRlist <- seq(1/sRmax, 1/sRmin, length.out=nsigmaR)
sigmaRlist <- seq(sRmin, sRmax, length.out=nsigmaR);
sigmaSlist/sigmaRlist
 
# Its actual value does not matter for uncontrainted architecture
# (rescaling and reducing the number of parameters)

# Discretization: number of possible phenotypes
# The more, the better
nptslist <- c(501);

# Relative proportion of each peak
# Is equal to 1/2 in TPH
theq <- 2/3;

# Generate an array of all possible parameter values
#  Note: because of the unconstrained architecture, the value of sigmaR does not matter (scaling)
cat('Generating the table of all parameters...')
parmslist <- expand.grid(deltatheta=deltalist, sigmaS=sigmaSlist, sigmaR=sigmaRlist, npts=nptslist, constrain=theconstrain, xm=1, q=theq)
cat(' done', '\n')


## COMPUTE THE MODE OF THE PERCEIVED DISTRIBUTION
#  Because the formula in the paper only works for q=1/2

# Compute the mode index
cat('Computing the modes of the perceived distributions...')
	bim <- mclapply(seq_len(nrow(parmslist)), function(i) do.call(is.bimod, list(prms=parmslist[i,])), mc.cores=cores)
cat(' done \n')
nx <- length(unique(parmslist$deltatheta));
ny <- length(unique(parmslist$sigmaR));

# Extract the countour lines data
mbim <- matrix(unlist(bim), nc=ny, byrow=FALSE)
xlist <- parmslist$deltatheta/parmslist$sigmaS
ylist <- parmslist$sigmaR/parmslist$sigmaS

xx <- sort(unique(parmslist$deltatheta/parmslist$sigmaS))
yy <- sort(unique(parmslist$sigmaR/parmslist$sigmaS))
cL <- contourLines(xx, yy, mbim, levels=.5)

# Export the results
save(parmslist, bim, mbim, cL, file='data/clonalgrid.Rdata')
load('data/clonalgrid.Rdata')
cl <- cL[[1]];

## PLOTTING
# Color codes
colS <- gray(0.35);	# No diversification because of stabilizing selection
colDM <- gray(0.55); 	# Diversification but unimodal equilibrium distribution
colDD <- gray(1);	# Diversification with Delta Peaks
colDG <- gray(0.9); 	# Diversification with Gaussian functions

lwd1 <- 3; 	# Line width for X and Y
lwd2 <- 1; # Line width for info lines


# Labels information
yb <- 3.8; dx <- 0.75; cx <- 1.25;

sS <- parmslist$sigmaS[1];

pdf('figures/fig1A.pdf', width=4, height=4)

	par(mar=c(5,5,0,0)+0.2, las=1)

	# Initiate the plot
	plot(c(dmin, dmax)/sS, c(sRmin, sRmax)/sS, col='white', pch=20, xlab=expression(paste(Delta, theta,'/',sigma[S])), ylab=expression(paste(sigma[R],'/',sigma[S])), cex.lab=1.4, cex.axis=1.2, ylim=c(0,sRmax/sS), axes=FALSE)

	# Plot lines for Delta Peak vs Gaussian distributions
	polygon(c(dmin, dmax, dmax, dmin, dmin)/sS, c(1, 1, sRmax/sS, sRmax/sS, 1), col=colDG, border=colDG)
	lines(c(dmin, dmax)/sigmaSlist, c(1,1), col=1, lty=3, lwd=lwd2)

	# Plot curve for felt bimodality
	polygon(c(0, cl$x, dmax/sS, 0, 0), c(sRmax/sS, cl$y, sRmax/sS, sRmax/sS, cl$y[1]), col=colDM, border=colDM)
	lines(cl$x, cl$y, col=1, lty=2, lwd=lwd1)
	
		# Plot line for explicit resource modality
	# -> Expression only works for q=1/2, so only plot it in this case
	if(sum(parmslist$q==1/2)>0){
		lines(rep(2,2),c(sSmin,sSmax)/sigmaRlist,col='black', lty=2, lwd=lwd2)
	}

	# Plot curve for the disruptiveness of selection
	nxx <- 500;
	x <- seq(dmin/sS, sqrt(1/(theq*(1-theq))),length.out=nxx);
	polygon(c(x, sqrt(1/(theq*(1-theq))), x[1], x[1]), c(sqrt(1 - theq*(1-theq)*x^2), 0, 0, sqrt(1 - theq*(1-theq)*x[1])), border=colS, col=colS)
	lines(x, sqrt(1 - theq*(1-theq)*x^2), col='black', lwd=lwd1)

	# Legends: points for the simulations
	xsim <- c(0.5,1.85,3.75, 3.75, 0.5, 1.85);
	ysim <- c(rep(1.15,3),0.75, 0.75, 0.35);
	text(xsim+c(rep(0.175,5),0.1), ysim+c(rep(-0.035,5),-0.075), c('b','c','d', 'g', 'e', 'f'), col=gray(c(0,0,0,0,1,1)), cex=cx, bg=rgb(1,1,1,0.5));
	points(xsim, ysim, pch=4, cex=0.8, col=c(rep(1,4),rep('white',2)), lwd=2.5)

	# Legends: different areas
	xtext <- c(1.25, 3.25, 3.25, 1.25)
	ytext <- c(2, 1.3, 0.65, 0.4)
	text(xtext, ytext, c('D2m', 'D2p', 'D1', 'S'), col=c(gray(1), gray(0), gray(0), gray(1)), cex=1.15*cx, bg=rgb(1,1,1,0.5), font=3);

	# Add axes
	colsupptick <- gray(0.4)
	axis(1, pos=0, at=c(0), labels=c(''), lwd=0, lwd.ticks=3.5, tcl=-0.65, col=colsupptick)
	axis(2, pos=0, at=c(0), labels=c(''), lwd=0, lwd.ticks=3.5, tcl=-0.65, col=colsupptick)
	axis(1, pos=0, at=seq(0, 4, by=1), labels=c('0.0', '1.0', '2.0', '3.0', '4.0'))
	axis(2, pos=0, at=seq(0,2.5,by=0.5))

	# Add the classical limit cases
	axis(1, pos=-0.15, at=c(0), labels=c('Roughgarden'), lwd=0, lwd.ticks=0, col.axis=colsupptick)
	axis(2, pos=-0.3, at=c(0), labels=c('Levene '), lwd=0, col.axis=colsupptick)

dev.off()


