cat('\nNote: If there is an error message, make sure you evaluate fig1A.R first. \n\n')

## FUNCTIONS

# MODEL FUNCTIONS
# F(U,x)
foraging <- function(x, U, prms){
	return( exp( -(x-U)^2/(2*prms$sigmaS^2) ) )
}

# R(x)
resources <- function(x, prms){
	return( (prms$q*exp(-(x-prms$deltatheta/2)^2/(2*prms$sigmaR^2)) + (1-prms$q)*exp(-(x+prms$deltatheta/2)^2/(2*prms$sigmaR^2))))
}

# FUNCTIONS FOR RUNNING SIMULATIONS
xmscale <- 6;

run.sim <- function(prms){
	# Initializations
	if(prms$constrain){	# Extreme genotypes
		xm <- prms$xm	# Fixed, if constrained architecture
	}else{
		if(prms$xm== -10){
			xm <- xmscale*max(prms$sigmaS, prms$sigmaR);	# Changed if unconstrained
		}else{
			xm <- prms$xm	# Or use the user-defined value
		}
	}
	pts <- seq(-xm, xm, length.out=prms$npts);	# npts is the total number of phenotypes (the higher, the better, but the longer it takes!)
	xvmat <- rep(pts, prms$npts);
	Uvmat <- rep(pts, each=prms$npts);

	xmat <- matrix(xvmat, nc=prms$npts);	# Matrix of possible resource values (x traits)
	Umat <- matrix(Uvmat, nc=prms$npts);	# Matrix of possible trait values (x resources)

	FT <- foraging(xmat, Umat, prms);	# Precompute all possible foraging values, for all (x,U) couples
	RT <- resources(xmat, prms); RT <- RT/sum(RT[,1]);	# Precompute all possible resource availabilities
	FRT <- FT*RT;	# All possible available resources for all possible phenotypes

	inits <- matrix(rep(1/prms$npts, prms$npts^2),nc=prms$npts);	# Initial state: all traits are present in the same frequency

	# SIMULATION
	p <- inits;

	for(i in 2:prms$nt){
		# Write the recursions
		meanF <- matrix(rep(rowSums(FT*p),prms$npts), nc=prms$npts);
		meanF[meanF<=0] <- 1;	# In case meanF=0, because we divide by meanF
		newp <- colSums((FRT/meanF)*p);	# Sum over all resources the relative resource intakes of each phenotype, times their frequency at time t
		p <- matrix(rep(newp, each=prms$npts), nc=prms$npts)	# Make the vector of phenotype frequencies into a matrix
	}
	write.table(matrix(c(prms$deltatheta, prms$sigmaR, prms$sigmaS, newp),nr=1), file=toString(prms$name), append=TRUE, row.names=F, col.names=F);
	return(newp)
}

# FUNCTION FOR PLOTTING THE RESULTS
# Parameters
# res   final distribution (output of run.sim)
# prms  list of parameters
# dev   device for plotting
# name  name of the output file
# xm    extreme phenotype values, if contrained
# dots  whether to plot dots over the lines

plot.sim <- function(res, prms, dev='X11', name='', xm=-10, dots=FALSE){
	if(dev=='pdf'){pdf(width=4, height=3, file=name)}
	par(mar=c(4.5,5,0,0)+0.2, las=1, mgp=c(2,1,0))
	if (xm == -10){
		xm <- xmscale*max(prms$sigmaS, prms$sigmaR);
	}
	pts <- seq(-xm, xm, length.out=prms$npts);
	RT <- resources(pts, prms); RT <- RT/max(RT);
	res <- res/max(res);

	if(dots){thetype <- 'o'}else{thetype='l'}
	plot(pts, res, type=thetype, ylim=c(0,max(c(RT, res))), ylab='Frequency', xlab='Trait/resource', col=gray(0), lty=1, lwd=1.5, cex.lab=1.4, cex.axis=1.2, pch=19, cex=0.5, axes=FALSE); 
	polygon(c(pts,pts[3]), c(RT,RT[3]), col=gray(0.8), border='white');
	lines(pts, res, lwd=1.5);
	axis(1, pos=0, at=c(pts[3], -prms$deltatheta/2, prms$deltatheta/2, pts[prms$npts]), labels=c('', expression(theta[1]), expression(theta[2]), ''), cex.axis=1.2);
	axis(2, at=c(0,1), labels=c('',''), pos=pts[3]);
	if(dev=='pdf') dev.off()
}


## RUN THE SIMULATIONS
theq <- 2/3;
thent <- 1000;
thenpts <- 501;
thexm <- 0.75;
sS <- 0.1;

cat('Simulations running... Out of 6: ')
# Fig 1B
theparms1 <- list(deltatheta=xsim[1]*sS, sigmaS=sS, sigmaR=ysim[1]*sS, npts=thenpts, nt=thent, q=theq, name='tmp.txt', constrain=FALSE, xm=thexm);
out1 <- run.sim(theparms1);
cat('1-')

# Fig 1C
theparms2 <- list(deltatheta=xsim[2]*sS, sigmaS=sS, sigmaR=ysim[2]*sS, npts=thenpts, nt=thent, q=theq, name='tmp.txt', constrain=FALSE, xm=thexm);
out2 <- run.sim(theparms2);
cat('2-')

# Fig 1D
theparms3 <- list(deltatheta=xsim[3]*sS, sigmaS=sS, sigmaR=ysim[3]*sS, npts=thenpts, nt=thent, q=theq, name='tmp.txt', constrain=FALSE, xm=thexm);
out3 <- run.sim(theparms3);
cat('3-')

# Fig 1E
theparms4 <- list(deltatheta=xsim[4]*sS, sigmaS=sS, sigmaR=ysim[4]*sS, npts=thenpts, nt=thent, q=theq, name='tmp.txt', constrain=FALSE, xm=thexm);
out4 <- run.sim(theparms4);
cat('4-')

# Fig 1F
theparms5 <- list(deltatheta=xsim[5]*sS, sigmaS=sS, sigmaR=ysim[5]*sS, npts=thenpts, nt=thent, q=theq, name='tmp.txt', constrain=FALSE, xm=thexm);
out5 <- run.sim(theparms5);
cat('5-')

# Fig 1G
theparms6 <- list(deltatheta=xsim[6]*sS, sigmaS=sS, sigmaR=ysim[6]*sS, npts=thenpts, nt=thent, q=theq, name='tmp.txt', constrain=FALSE, xm=thexm);
out6 <- run.sim(theparms6);
cat('6. Done! now plotting the outcome\n')

## PLOT THE OUTCOME
plot.sim(out1, theparms1, dev='pdf', name='figures/fig1B.pdf', xm=thexm);
plot.sim(out2, theparms2, dev='pdf', name='figures/fig1C.pdf', xm=thexm);
plot.sim(out3, theparms3, dev='pdf', name='figures/fig1D.pdf', xm=thexm);
plot.sim(out4, theparms4, dev='pdf', name='figures/fig1G.pdf', xm=thexm);
plot.sim(out5, theparms5, dev='pdf', name='figures/fig1E.pdf', xm=thexm);
plot.sim(out6, theparms6, dev='pdf', name='figures/fig1F.pdf', xm=thexm);

