# Clear the memory
rm(list=ls())
# Close all figure windows
for (i in dev.list()) dev.off()

## GLOBAL PARAMETERS 
cores <- 2;	# Number of cores that will be used

# Arrays of parameters
pts <- 101 # Number of possibles values
# there will be pts^3 evaluations (and this will take time!)

# \Delta \theta
ndelta <- pts;	# Number of possible values
dmax <- 10;
dmin <- 0.00001;
deltalist <- seq(0, dmax, length.out=ndelta);
deltalist[1] <- dmin

# \sigma_S
sigmaSlist <- c(1);

# \sigma_R
nsigmaR <- pts;	# Number of possible values
sRmax <- 1;
sRmin <- 0.00001;
sigmaRlist <- seq(0, sRmax, length.out=nsigmaR);
sigmaRlist[1] <- sRmin
 
# Discretization
nptslist <- c(501);

# Relative proportion of each peak (q)
# Is equal to 1/2 in TPH
nq <- pts;
qlist <- seq(0, 1, length.out=nq)
 epsilonq <- 1/(2*nq); # Small value (we do not want q=0 or q=1)
 qlist[1] <- epsilonq
 qlist[nq] <- 1- epsilonq

# Maximal relative value of Um which will be screened
# (factor in front of deltatheta)
fac <- 2

# Generate an array of all possible parameter values
cat('Generating the table of all parameters...')
parmslist <- expand.grid(deltatheta=deltalist, sigmaS=sigmaSlist, sigmaR=sigmaRlist, npts=nptslist, xm=1, q=qlist, fac=fac)
cat(' done', '\n')

# Function which returns TRUE if selection is stabilizing (locally)
is.stab <- function(parms){
	Y <- parms$deltatheta^2 + 1/(parms$q * (1-parms$q)) * (parms$sigmaR^2 - parms$sigmaS^2)
	if(Y < 0 ) out <- TRUE else out <- FALSE
	return(out)
}

# Function which return the invasion fitness wm of individuals with trait Um 
# when the resident is at the singular strategy
invasionfitness <- function(parms, Um){
	return(exp( ((Um - parms$deltatheta / 2 * (1 - 2 * parms$q))^2 * parms$sigmaR^2 - (Um + parms$deltatheta/2 * (1-2*parms$q)) * ( Um + parms$deltatheta/2 * (1+2*parms$q)) * parms$sigmaS^2 )/ (2 * parms$sigmaS^4) ) * ( exp( Um * parms$deltatheta / parms$sigmaS^2 ) * (1-parms$q) + exp( parms$deltatheta^2 * (1 - 2*parms$q) / (2*parms$sigmaS^2)) * parms$q))
}

# Function to determine which is the maximal value of wm, for a vector of Um values
# The maximum is 1 when selection is globally stabilizing, and is attained when Um = \bar{x}
# If there is another peak, then it means that there are bistabilities
checkmaxinv <- function(parms){
	if(is.stab(parms)){ # When selection is locally stabilizing
		# List of Um values
		Umlist <- seq(- parms$fac * parms$deltatheta, parms$fac * parms$deltatheta, length.out = parms$npts)
		# Return the max invasion fitness for these Um values
		out <- max(invasionfitness(parms, Umlist))
	}else{ # If selection is not locally stabilizing, it won't be globally
		out <- -1
	}
	return(out)
}

# Compute all max invasion fitnesses for all parameters
# THIS TAKES TIME!
# Load the multicore package to parallelize the simulations
require(multicore)
cat('Computing the maximum value of the invasion fitness...')
	wmax <- mclapply(seq_len(nrow(parmslist)), function(i) do.call(checkmaxinv, list(parmslist[i,])), mc.cores=cores)
cat(' done \n')

# Save the results
tab <- cbind(parmslist, wmax=unlist(wmax))
save(tab, file=paste('data',pts,'.RData', sep=''))

# Reload the results
load(paste('data',pts,'.RData', sep=''))
parmslist <- tab

thr <- 1.00001 # Threshold for the max value

# Functions to analyze the results
# Proportion of the simulations with a bistability
nbsupp <- function(v){
	sum(v>thr) / length(v)
	}

# Proportion of the simulations with a bistability, 
# among these with stabilizing selection
nbsuppnodiv <- function(v){
	ndiv <- sum(v>=0)
	if(ndiv>0) out <- sum(v>thr) / sum(v>=0) else out <- -1
	return(out)
	}

# Proportion of the simulations with disruptive selection
nbdiv <- function(v){
	sum(v<0) / length(v)
	}

# Aggregate the results, along the axes deltatheta and q
cat('Aggregating data...')
wdq <- aggregate(tab, by=list(tab$deltatheta, tab$q), nbsuppnodiv)
divdq <- aggregate(tab, by=list(tab$deltatheta, tab$q), nbdiv)
cat(' done \n')

# Extract contour lines data for the area with disruptive selection
cL <- contourLines(sort(unique(wdq[,1])), sort(unique(wdq[,2])),matrix(wdq$wmax, ncol=length(unique(wdq[,1]))), levels=c(-0.5))
cl <- cL[[1]]

## PLOT THE RESULTS
pdf('figures/figS4.pdf', width=5.75, height=3.)
	par(mar=c(3.5, 4, 0, 0)+0.5, las=1)

	# Plotting parameters
	bby <- 0.05 # distance btw two colors
	listbreak <- seq(0.0,1,by=bby) # List of levels for contour plot
	rdis <- 0 # Red level of the area with disruptive selection
	gdis <- 0 # Green
	bdis <- 0 # Blue
	opdis <- 0.15 # Opacity
	cextxt <- 0.9 # Relative size of text

	# Plot
	image(sort(unique(wdq[,1])), sort(unique(wdq[,2])),matrix(wdq$wmax, ncol=length(unique(wdq[,1]))), breaks=c(-1.5, -0.5, listbreak+bby/2), col=c('white',rgb(1, 0, 0, listbreak)), xlab='', ylab=expression(paste('Rel. prop. of resource ', theta[1], '(',q, ')')), axes=FALSE, ylim=c(0,1))

	# Add color for area with disruptive selection
	polygon(cl$x, cl$y, col=rgb(rdis, gdis, bdis, opdis), border=NA)
	lines(cl$x, cl$y, type='l', lwd=3, col=rgb(rdis, gdis, bdis))

	# Add labels
	text(6, 0.5, 'Locally\ndisruptive selection', cex=cextxt)
	text(0.85, 0.5, 'Locally\nstabilizing\n selection', cex=cextxt)

	# Plot axes
	axis(1, pos=0)
	axis(2, at=seq(0,1,by=0.2), pos=0)
	axis(3, lwd.ticks=-1)
	mtext(expression(paste('Distance between the peaks (', Delta,theta, ')')), 1, line=2.25)
dev.off()

## PLOT THE LEGEND
pdf('figures/figS4legend.pdf', width=1.65, height=3.)
	par(mar=c(0.25, 0, 0.25, 5.5)+0.5, las=1)

	image(1, c(-listbreak[2]/2,listbreak), matrix(c(-listbreak[2]/2, listbreak), nrow=1), col=rgb(1, 0, 0, listbreak), xlab='', ylab='', axes=FALSE)

	axis(4, at=c(0, 1), labels=c('Globally\nstabilizing', 'Not globally\nstabilizing'), lwd=0)
dev.off()


